@@ -212,6 +212,12 @@ color_balance_initialize (GDisplay *gdisp)
 {
   gint i;
 
+  if (gimp_image_is_empty (gdisp->gimage))
+    {
+      g_message (_("The image has no drawables."));
+      return;
+    }
+
   if (! drawable_color (gimage_active_drawable (gdisp->gimage)))
     {
       g_message (_("Color balance operates only on RGB color drawables."));
@@ -237,6 +243,13 @@ color_balance_initialize (GDisplay *gdisp)
     image_map_create (gdisp, color_balance_dialog->drawable);
 
   color_balance_update (color_balance_dialog, ALL);
+
+  /* Merge-related undo release signal */
+
+  color_balance_dialog->conn_id = 
+    gtk_signal_connect (GTK_OBJECT (gdisp->gimage), "layer_merge",
+			GTK_SIGNAL_FUNC (color_balance_cancel_callback),
+			color_balance_dialog);
 }
 
 /**************************/
@@ -424,7 +437,7 @@ color_balance_dialog_new (void)
       group = gtk_radio_button_group (GTK_RADIO_BUTTON (radio_button));
       gtk_box_pack_start (GTK_BOX (hbox), radio_button, TRUE, FALSE, 0);
 
-      gtk_object_set_user_data (GTK_OBJECT (radio_button), (gpointer) i);
+      gtk_object_set_user_data (GTK_OBJECT (radio_button), GINT_TO_POINTER (i));
       gtk_signal_connect (GTK_OBJECT (radio_button), "toggled",
 			  GTK_SIGNAL_FUNC (color_balance_range_callback),
 			  cbd);
@@ -618,6 +631,12 @@ color_balance_cancel_callback (GtkWidget *widget,
       cbd->image_map = NULL;
     }
 
+  if (cbd->conn_id != 0)
+    {
+      gtk_signal_disconnect (GTK_OBJECT (gimp_drawable_gimage (cbd->drawable)), cbd->conn_id);
+      cbd->conn_id = 0;
+    }
+
   active_tool->gdisp_ptr = NULL;
   active_tool->drawable = NULL;
 }
