@@ -15,6 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
 #include "config.h"
 
 #include <stdlib.h>
@@ -30,7 +31,9 @@
 #include <sys/param.h>
 #endif
 
-#include <glib.h>
+#include <gtk/gtk.h>
+
+#include "apptypes.h"
 
 #include "app_procs.h"
 #include "appenv.h"
@@ -59,6 +62,9 @@
 #define OK         2
 #define LOCALE_DEF 3
 #define HELP_DEF   4
+#ifdef HACK_FOR_BUG_66859
+#define INIT_DEF   5
+#endif
 
 typedef enum
 {
@@ -372,7 +378,7 @@ gimp_system_rc_file (void)
   if (value != NULL)
     return value;
 
-  value = g_strconcat (gimp_data_directory (), G_DIR_SEPARATOR_S,
+  value = g_strconcat (gimp_sysconf_directory (), G_DIR_SEPARATOR_S,
 		       "gimprc", NULL);
   return value;
 }
@@ -420,8 +426,9 @@ parse_gimprc (void)
   else
     libfilename = g_strdup (gimp_system_rc_file ());
     
-  parse_gimprc_file (libfilename);
-
+  if (! parse_gimprc_file (libfilename))
+    g_message ("Can't open '%s' for reading.", libfilename);
+  
   if (alternate_gimprc != NULL) 
     filename = g_strdup (alternate_gimprc);
   else 
@@ -486,10 +493,17 @@ parse_gimprc_file (gchar *filename)
 
   if (!g_path_is_absolute (filename))
     {
-      if (g_get_home_dir () != NULL)
+      gchar *home_dir = g_get_home_dir ();
+      gchar *home_dir_sep;
+
+      if (home_dir != NULL)
 	{
-	  rfilename = g_strdup_printf ("%s" G_DIR_SEPARATOR_S "%s",
-				       g_get_home_dir (), filename);
+	  if (home_dir[strlen (home_dir) -1] != G_DIR_SEPARATOR)
+	    home_dir_sep = G_DIR_SEPARATOR_S;
+	  else
+	    home_dir_sep = "";
+	  rfilename = g_strdup_printf ("%s%s%s", home_dir, home_dir_sep,
+				       filename);
 	  parsed = parse_absolute_gimprc_file (rfilename);
 	  g_free (rfilename);
 	  return parsed;
@@ -741,9 +755,8 @@ save_gimprc (GList **updated_options,
 	      g_free (str);
 
 	      *updated_options = g_list_remove_link (*updated_options, option);
-	      *conflicting_options = g_list_append (*conflicting_options,
-						    (gpointer) option->data);
-	      g_list_free_1 (option);
+	      *conflicting_options = g_list_concat (*conflicting_options,
+						    option);
 	      continue;
 	    }
 
@@ -1427,6 +1440,16 @@ parse_plug_in_def (gpointer val1p,
 	success = parse_locale_def (plug_in_def);
       else if (success == HELP_DEF)
 	success = parse_help_def (plug_in_def);
+#ifdef HACK_FOR_BUG_66859
+      else if (success == INIT_DEF)
+	{
+	  token = peek_next_token ();
+	  if (!token || token != TOKEN_RIGHT_PAREN)
+	    goto error;
+	  token = get_next_token ();
+	  plug_in_def->init = TRUE;
+	}
+#endif
     }
 
   token = peek_next_token ();
@@ -1540,7 +1563,14 @@ parse_proc_def (PlugInProcDef **proc_def)
       token = get_next_token ();
       return HELP_DEF;  /* it's a help_def, let parse_help_def do the rest */
     }
-  else if (strcmp ("proc-def", token_sym) != 0)
+#ifdef HACK_FOR_BUG_66859
+  if ((strcmp ("has-init", token_sym) == 0))
+    {
+      token = get_next_token ();
+      return INIT_DEF;  /* let parse_plug_in_def do the rest */
+    }
+#endif
+  if (strcmp ("proc-def", token_sym) != 0)
     return ERROR;
 
   token = get_next_token ();
