@@ -15,8 +15,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
 #include "config.h"
 
+#include <gtk/gtk.h>
+
+#include "apptypes.h"
+
 #include "appenv.h"
 #include "gdisplay.h"
 #include "resize.h"
@@ -66,19 +71,41 @@ struct _ResizePrivate
   gint    orig_x, orig_y;
 };
 
-static void  resize_draw                 (Resize *);
-static void  unit_update                 (GtkWidget *, gpointer);
-static gint  resize_bound_off_x          (Resize *, gint);
-static gint  resize_bound_off_y          (Resize *, gint);
-static void  orig_labels_update          (GtkWidget *, gpointer);
-static void  size_callback               (GtkWidget *, gpointer);
-static void  ratio_callback              (GtkWidget *, gpointer);
-static void  size_update                 (Resize *, gdouble, gdouble, gdouble, gdouble);
-static void  offset_update               (GtkWidget *, gpointer);
-static gint  resize_events               (GtkWidget *, GdkEvent *);
-static void  printsize_update            (GtkWidget *, gpointer);
-static void  resolution_update           (GtkWidget *, gpointer);
-static void  resize_scale_warn_callback  (GtkWidget *, gboolean, gpointer);
+static void  resize_draw                 (Resize    *resize);
+static void  unit_update                 (GtkWidget *widget,
+					  gpointer   data);
+static gint  resize_bound_off_x          (Resize    *resize,
+					  gint       off_x);
+static gint  resize_bound_off_y          (Resize    *resize,
+					  gint       off_y);
+static void  orig_labels_update          (GtkWidget *widget,
+					  gpointer   data);
+static void  reset_callback              (GtkWidget *widget,
+					  gpointer   data);
+static void  size_callback               (GtkWidget *widget,
+					  gpointer   data);
+static void  ratio_callback              (GtkWidget *widget,
+					  gpointer   data);
+static void  size_update                 (Resize    *widget,
+					  gdouble    width,
+					  gdouble    height,
+					  gdouble    ratio_x,
+					  gdouble    ratio_y);
+static void  offset_update               (GtkWidget *widget,
+					  gpointer   data);
+static gint  resize_events               (GtkWidget *widget,
+					  GdkEvent  *event);
+static void  printsize_update            (GtkWidget *widget,
+					  gpointer   data);
+static void  resolution_callback         (GtkWidget *widget,
+					  gpointer   data);
+static void  resolution_update           (Resize    *resize,
+					  gdouble    res_x,
+					  gdouble    res_y);
+static void  resize_scale_warn_callback  (GtkWidget *widget,
+					  gboolean   do_scale,
+					  gpointer   data);
+
 
 Resize *
 resize_widget_new (ResizeType    type,
@@ -110,7 +137,7 @@ resize_widget_new (ResizeType    type,
   abox = NULL;
   frame = NULL;
 
-  private = g_new (ResizePrivate, 1);
+  private = g_new0 (ResizePrivate, 1);
   private->old_width  = width;
   private->old_height = height;
   private->old_res_x  = resolution_x;
@@ -171,12 +198,12 @@ resize_widget_new (ResizeType    type,
 	  case ResizeLayer:
 	    wmclass = "resize_layer";
 	    window_title = _("Set Layer Boundary Size");
-	    help_page = "layers/dialogs/resize_layer.html";
+	    help_page = "layers/dialogs/layer_boundary_size.html";
 	    break;
 	  case ResizeImage:
 	    wmclass = "resize_image";
 	    window_title = _("Set Canvas Size");
-	    help_page = "dialogs/resize_image.html";
+	    help_page = "dialogs/set_canvas_size.html";
 	    break;
 	  }
 	frame = gtk_frame_new (_("Size"));
@@ -191,6 +218,10 @@ resize_widget_new (ResizeType    type,
 
 		       _("OK"), ok_cb,
 		       user_data, NULL, NULL, TRUE, FALSE,
+
+		       _("Reset"), reset_callback,
+		       resize, NULL, NULL, FALSE, FALSE,
+
 		       _("Cancel"), cancel_cb ? cancel_cb : gtk_widget_destroy,
 		       cancel_cb ? user_data : NULL,
 		       cancel_cb ? NULL : (gpointer) 1,
@@ -324,10 +355,10 @@ resize_widget_new (ResizeType    type,
   gimp_size_entry_set_refval (GIMP_SIZE_ENTRY (private->size_se), 1, height);
 
   gtk_signal_connect (GTK_OBJECT (private->size_se), "value_changed",
-		      (GtkSignalFunc) size_callback,
+		      GTK_SIGNAL_FUNC (size_callback),
 		      resize);
   gtk_signal_connect (GTK_OBJECT (private->size_se), "unit_changed",
-		      (GtkSignalFunc) orig_labels_update,
+		      GTK_SIGNAL_FUNC (orig_labels_update),
 		      resize);
 
   /*  initialize the original width & height labels  */
@@ -362,7 +393,8 @@ resize_widget_new (ResizeType    type,
 			(double) GIMP_MIN_IMAGE_SIZE / (double) resize->width,
 			(double) GIMP_MAX_IMAGE_SIZE / (double) resize->width,
 			0.01, 0.1, 1);
-  spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (private->ratio_x_adj), 1, 4);
+  spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (private->ratio_x_adj),
+                                    0.01, 4);
   gtk_spin_button_set_shadow_type (GTK_SPIN_BUTTON (spinbutton),
                                    GTK_SHADOW_NONE);
   gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton), TRUE);
@@ -378,7 +410,8 @@ resize_widget_new (ResizeType    type,
 			(double) GIMP_MIN_IMAGE_SIZE / (double) resize->height,
 			(double) GIMP_MAX_IMAGE_SIZE / (double) resize->height,
 			0.01, 0.1, 1);
-  spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (private->ratio_y_adj), 1, 4);
+  spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (private->ratio_y_adj),
+                                    0.01, 4);
   gtk_spin_button_set_shadow_type (GTK_SPIN_BUTTON (spinbutton),
                                    GTK_SHADOW_NONE);
   gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton), TRUE);
@@ -455,7 +488,8 @@ resize_widget_new (ResizeType    type,
       gimp_size_entry_set_refval (GIMP_SIZE_ENTRY (private->offset_se), 1, 0);
 
       gtk_signal_connect (GTK_OBJECT (private->offset_se), "value_changed",
-			  (GtkSignalFunc) offset_update, resize);
+			  GTK_SIGNAL_FUNC (offset_update),
+			  resize);
 
       gtk_widget_show (abox);
 
@@ -472,7 +506,7 @@ resize_widget_new (ResizeType    type,
 			     private->area_width, private->area_height);
       gtk_widget_set_events (private->drawing_area, EVENT_MASK);
       gtk_signal_connect (GTK_OBJECT (private->drawing_area), "event",
-			  (GtkSignalFunc) resize_events,
+			  GTK_SIGNAL_FUNC (resize_events),
 			  NULL);
       gtk_object_set_user_data (GTK_OBJECT (private->drawing_area), resize);
       gtk_container_add (GTK_CONTAINER (frame), private->drawing_area);
@@ -556,10 +590,10 @@ resize_widget_new (ResizeType    type,
 				  1, resize->height);
 
       gtk_signal_connect (GTK_OBJECT (private->printsize_se), "value_changed",
-			  (GtkSignalFunc) printsize_update,
+			  GTK_SIGNAL_FUNC (printsize_update),
 			  resize);
       gtk_signal_connect (GTK_OBJECT (private->printsize_se), "unit_changed",
-			  (GtkSignalFunc) unit_update,
+			  GTK_SIGNAL_FUNC (unit_update),
 			  resize);
       
       /*  the resolution labels  */
@@ -611,7 +645,7 @@ resize_widget_new (ResizeType    type,
 				  1, resize->resolution_y);
 
       gtk_signal_connect (GTK_OBJECT (private->resolution_se), "value_changed",
-			  (GtkSignalFunc) resolution_update,
+			  GTK_SIGNAL_FUNC (resolution_callback),
 			  resize);
 
       /*  the resolution chainbutton  */
@@ -824,12 +858,12 @@ offset_update (GtkWidget *widget,
   resize = (Resize *) data;
   private = (ResizePrivate *) resize->private_part;
 
-  offset_x = (gint)
-    (gimp_size_entry_get_refval (GIMP_SIZE_ENTRY (private->offset_se), 0) + 0.5);
+  offset_x =
+    RINT (gimp_size_entry_get_refval (GIMP_SIZE_ENTRY (private->offset_se), 0));
   offset_x = resize_bound_off_x (resize, offset_x);
 
-  offset_y = (gint)
-    (gimp_size_entry_get_refval (GIMP_SIZE_ENTRY (private->offset_se), 1) + 0.5);
+  offset_y =
+    RINT (gimp_size_entry_get_refval (GIMP_SIZE_ENTRY (private->offset_se), 1));
   offset_y = resize_bound_off_y (resize, offset_y);
 
   if ((offset_x != resize->offset_x) ||
@@ -841,38 +875,63 @@ offset_update (GtkWidget *widget,
     }
 }
 
+/*
+ * Callback function for "Reset" button.
+ * Data must be a pointer pointer to a Resize structure.
+ */
+static void
+reset_callback (GtkWidget *widget,
+                gpointer data)
+{
+  Resize *resize;
+  ResizePrivate *private;
+
+  resize = (Resize *)data;
+  private = (ResizePrivate *)resize->private_part;
+
+  /* restore size and ratio settings */
+  size_update (resize, private->old_width, private->old_height, 1.0, 1.0);
+
+  if ((resize->type == ScaleWidget) && (resize->target == ResizeImage))
+    {
+      /* restore resolution settings */
+      resolution_update (resize, private->old_res_x, private->old_res_y);
+    }
+
+}
+  
 static void
 size_callback (GtkWidget *widget,
 	       gpointer   data)
 {
   Resize *resize;
   ResizePrivate *private;
-  double width;
-  double height;
-  double ratio_x;
-  double ratio_y;
+  gdouble width;
+  gdouble height;
+  gdouble ratio_x;
+  gdouble ratio_y;
 
-  resize = (Resize *) data;
+  resize  = (Resize *) data;
   private = (ResizePrivate *) resize->private_part;
 
-  width = gimp_size_entry_get_refval (GIMP_SIZE_ENTRY (private->size_se), 0);
+  width  = gimp_size_entry_get_refval (GIMP_SIZE_ENTRY (private->size_se), 0);
   height = gimp_size_entry_get_refval (GIMP_SIZE_ENTRY (private->size_se), 1);
 
-  ratio_x = width / (double) private->old_width;
-  ratio_y = height / (double) private->old_height;
+  ratio_x = width  / (gdouble) private->old_width;
+  ratio_y = height / (gdouble) private->old_height;
 
   if (gimp_chain_button_get_active (GIMP_CHAIN_BUTTON (private->constrain)))
     {
       if (ratio_x != resize->ratio_x)
 	{
 	  ratio_y = ratio_x;
-	  height = (double) private->old_height * ratio_y;
+	  height = (gdouble) private->old_height * ratio_y;
 	  height = CLAMP (height, GIMP_MIN_IMAGE_SIZE, GIMP_MAX_IMAGE_SIZE);
 	}
       else
 	{
 	  ratio_x = ratio_y;
-	  width = (double) private->old_width * ratio_x;
+	  width = (gdouble) private->old_width * ratio_x;
 	  width = CLAMP (width, GIMP_MIN_IMAGE_SIZE, GIMP_MAX_IMAGE_SIZE);
 	}
     }
@@ -886,15 +945,15 @@ ratio_callback (GtkWidget *widget,
 {
   Resize *resize;
   ResizePrivate *private;
-  double width;
-  double height;
-  double ratio_x;
-  double ratio_y;
+  gdouble width;
+  gdouble height;
+  gdouble ratio_x;
+  gdouble ratio_y;
 
-  resize = (Resize *) data;
+  resize  = (Resize *) data;
   private = (ResizePrivate *) resize->private_part;
 
-  width = gimp_size_entry_get_refval (GIMP_SIZE_ENTRY (private->size_se), 0);
+  width  = gimp_size_entry_get_refval (GIMP_SIZE_ENTRY (private->size_se), 0);
   height = gimp_size_entry_get_refval (GIMP_SIZE_ENTRY (private->size_se), 1);
 
   ratio_x = GTK_ADJUSTMENT (private->ratio_x_adj)->value;
@@ -912,8 +971,8 @@ ratio_callback (GtkWidget *widget,
 	}
     }
 
-  width = CLAMP (private->old_width * ratio_x,
-		 GIMP_MIN_IMAGE_SIZE, GIMP_MAX_IMAGE_SIZE);
+  width  = CLAMP (private->old_width * ratio_x,
+		  GIMP_MIN_IMAGE_SIZE, GIMP_MAX_IMAGE_SIZE);
   height = CLAMP (private->old_height * ratio_y,
 		  GIMP_MIN_IMAGE_SIZE, GIMP_MAX_IMAGE_SIZE);
 
@@ -992,17 +1051,17 @@ printsize_update (GtkWidget *widget,
 {
   Resize *resize;
   ResizePrivate *private;
-  double width;
-  double height;
-  double print_width;
-  double print_height;
-  double res_x;
-  double res_y;
+  gdouble width;
+  gdouble height;
+  gdouble print_width;
+  gdouble print_height;
+  gdouble res_x;
+  gdouble res_y;
 
-  resize = (Resize *) data;
+  resize  = (Resize *) data;
   private = (ResizePrivate *) resize->private_part;
 
-  width = gimp_size_entry_get_refval (GIMP_SIZE_ENTRY (private->size_se), 0);
+  width  = gimp_size_entry_get_refval (GIMP_SIZE_ENTRY (private->size_se), 0);
   height = gimp_size_entry_get_refval (GIMP_SIZE_ENTRY (private->size_se), 1);
 
   print_width =
@@ -1066,16 +1125,17 @@ printsize_update (GtkWidget *widget,
 				      resize);
 }
 
+/* Callback for resolution change. */
 static void
-resolution_update (GtkWidget *widget,
-		   gpointer   data)
+resolution_callback (GtkWidget *widget,
+		     gpointer   data)
 {
   Resize *resize;
   ResizePrivate *private;
-  double res_x;
-  double res_y;
+  gdouble res_x;
+  gdouble res_y;
   
-  resize = (Resize *) data;
+  resize  = (Resize *) data;
   private = (ResizePrivate *) resize->private_part;
 
   res_x =
@@ -1095,6 +1155,19 @@ resolution_update (GtkWidget *widget,
 	}
     }
 
+  resolution_update (resize, res_x, res_y);
+}
+
+/* Update widgets with resolution settings found in given Resize struct. */
+static void
+resolution_update (Resize  *resize,
+		   gdouble  res_x,
+		   gdouble  res_y)
+{
+  ResizePrivate *private;
+
+  private = (ResizePrivate *) resize->private_part;
+
   resize->resolution_x = res_x;
   resize->resolution_y = res_y;
 
@@ -1128,10 +1201,10 @@ resize_events (GtkWidget *widget,
 {
   Resize *resize;
   ResizePrivate *private;
-  int dx, dy;
-  int off_x, off_y;
+  gint dx, dy;
+  gint off_x, off_y;
 
-  resize = (Resize *) gtk_object_get_user_data (GTK_OBJECT (widget));
+  resize  = (Resize *) gtk_object_get_user_data (GTK_OBJECT (widget));
   private = (ResizePrivate *) resize->private_part;
 
   switch (event->type)
@@ -1144,8 +1217,8 @@ resize_events (GtkWidget *widget,
 			(GDK_BUTTON1_MOTION_MASK |
 			 GDK_BUTTON_RELEASE_MASK),
 			NULL, NULL, event->button.time);
-      private->orig_x = resize->offset_x;
-      private->orig_y = resize->offset_y;
+      private->orig_x  = resize->offset_x;
+      private->orig_y  = resize->offset_y;
       private->start_x = event->button.x;
       private->start_y = event->button.y;
       break;
@@ -1182,15 +1255,16 @@ resize_events (GtkWidget *widget,
 void
 resize_scale_implement (ImageResize *image_scale)
 {
-  GImage      *gimage        = NULL;
-  gboolean     rulers_flush  = FALSE;
-  gboolean     display_flush = FALSE;  /* this is a bit ugly: 
-				          we hijack the flush variable 
-					  to check if an undo_group was 
-					  already started */
-  g_assert(image_scale != NULL);
+  GImage   *gimage        = NULL;
+  gboolean  rulers_flush  = FALSE;
+  gboolean  display_flush = FALSE;  /* this is a bit ugly: 
+				       we hijack the flush variable 
+				       to check if an undo_group was 
+				       already started */
+
+  g_assert (image_scale != NULL);
   gimage = image_scale->gimage;
-  g_assert(gimage != NULL);
+  g_assert (gimage != NULL);
 
   if (image_scale->resize->resolution_x != gimage->xresolution ||
       image_scale->resize->resolution_y != gimage->yresolution)
@@ -1257,13 +1331,13 @@ resize_scale_implement (ImageResize *image_scale)
 static void
 resize_scale_warn_callback (GtkWidget *widget,
 			    gboolean   do_scale,
-			    gpointer   client_data)
+			    gpointer   data)
 {
   ImageResize *image_scale = NULL;
   GImage      *gimage      = NULL;
 
-  g_assert (client_data != NULL);
-  image_scale = (ImageResize *) client_data;
+  g_assert (data != NULL);
+  image_scale = (ImageResize *) data;
   gimage      = image_scale->gimage;
   g_assert (gimage != NULL);
 
@@ -1333,4 +1407,3 @@ resize_check_layer_scaling (ImageResize *image_scale)
     }
   return success;
 }
-
