@@ -23,7 +23,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: waves.c,v 1.15 2000/08/20 20:34:26 austin Exp $
+ * $Id: waves.c,v 1.17.2.1 2003/01/15 03:16:09 yosh Exp $
  */
 
 #include "config.h"
@@ -86,7 +86,7 @@ static gint do_preview = TRUE;
 
 static GtkWidget        * mw_preview_new   (GtkWidget        *parent,
                                             struct mwPreview *mwp);
-static struct mwPreview * mw_preview_build (GDrawable        *drawable);
+static struct mwPreview * mw_preview_build (GimpDrawable        *drawable);
 
 static struct mwPreview *mwp;
 
@@ -94,9 +94,9 @@ static struct mwPreview *mwp;
 static void query (void);
 static void run   (gchar   *name,
 		   gint     nparam,
-		   GParam  *param,
+		   GimpParam  *param,
 		   gint    *nretvals,
-		   GParam **retvals);
+		   GimpParam **retvals);
 
 static gint pluginCore       (struct piArgs *argp,
 			      gint32         drawable);
@@ -123,7 +123,7 @@ static guchar bilinear (gdouble x,
 
 #define WITHIN(a, b, c) ((((a) <= (b)) && ((b) <= (c))) ? TRUE : FALSE)
 
-GPlugInInfo PLUG_IN_INFO =
+GimpPlugInInfo PLUG_IN_INFO =
 {
   NULL,  /* init_proc  */
   NULL,  /* quit_proc  */
@@ -136,16 +136,16 @@ MAIN ()
 static void
 query (void)
 {
-  static GParamDef args[] =
+  static GimpParamDef args[] =
   {
-    { PARAM_INT32, "run_mode", "Interactive, non-interactive" },
-    { PARAM_IMAGE, "image", "The Image" },
-    { PARAM_DRAWABLE, "drawable", "The Drawable" },
-    { PARAM_FLOAT, "amplitude", "The Amplitude of the Waves" },
-    { PARAM_FLOAT, "phase", "The Phase of the Waves" },
-    { PARAM_FLOAT, "wavelength", "The Wavelength of the Waves" },
-    { PARAM_INT32, "type", "Type of waves, black/smeared" },
-    { PARAM_INT32, "reflective", "Use Reflection" }
+    { GIMP_PDB_INT32, "run_mode", "Interactive, non-interactive" },
+    { GIMP_PDB_IMAGE, "image", "The Image" },
+    { GIMP_PDB_DRAWABLE, "drawable", "The Drawable" },
+    { GIMP_PDB_FLOAT, "amplitude", "The Amplitude of the Waves" },
+    { GIMP_PDB_FLOAT, "phase", "The Phase of the Waves" },
+    { GIMP_PDB_FLOAT, "wavelength", "The Wavelength of the Waves" },
+    { GIMP_PDB_INT32, "type", "Type of waves, black/smeared" },
+    { GIMP_PDB_INT32, "reflective", "Use Reflection" }
   };
   static gint nargs = sizeof (args) / sizeof (args[0]);
 
@@ -157,7 +157,7 @@ query (void)
 			  "1997",
 			  N_("<Image>/Filters/Distorts/Waves..."),
 			  "RGB*, GRAY*",
-			  PROC_PLUG_IN,
+			  GIMP_PLUGIN,
 			  nargs, 0,
 			  args, NULL);
 }
@@ -165,11 +165,11 @@ query (void)
 static void
 run (gchar   *name,
      gint     nparam,
-     GParam  *param,
+     GimpParam  *param,
      gint    *nretvals,
-     GParam **retvals)
+     GimpParam **retvals)
 {
-  static GParam rvals[1];
+  static GimpParam rvals[1];
 
   struct piArgs args;
 
@@ -180,13 +180,13 @@ run (gchar   *name,
   args.type = -1;
   gimp_get_data ("plug_in_waves", &args);
 
-  rvals[0].type = PARAM_STATUS;
-  rvals[0].data.d_status = STATUS_SUCCESS;
+  rvals[0].type = GIMP_PDB_STATUS;
+  rvals[0].data.d_status = GIMP_PDB_SUCCESS;
   switch (param[0].data.d_int32)
     {
-      GDrawable *drawable;
+      GimpDrawable *drawable;
 
-    case RUN_INTERACTIVE:
+    case GIMP_RUN_INTERACTIVE:
       INIT_I18N_UI();
       /* XXX: add code here for interactive running */
       if (args.type == -1)
@@ -203,7 +203,7 @@ run (gchar   *name,
 
       if (pluginCoreIA(&args, param[2].data.d_drawable) == -1)
 	{
-	  rvals[0].data.d_status = STATUS_EXECUTION_ERROR;
+	  rvals[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;
 	}
       else
 	{
@@ -212,12 +212,12 @@ run (gchar   *name,
 
     break;
 
-    case RUN_NONINTERACTIVE:
+    case GIMP_RUN_NONINTERACTIVE:
       INIT_I18N();
       /* XXX: add code here for non-interactive running */
       if (nparam != 8)
 	{
-	  rvals[0].data.d_status = STATUS_CALLING_ERROR;
+	  rvals[0].data.d_status = GIMP_PDB_CALLING_ERROR;
 	  break;
 	}
       args.amplitude  = param[3].data.d_float;
@@ -228,17 +228,17 @@ run (gchar   *name,
 
       if (pluginCore (&args, param[2].data.d_drawable) == -1)
 	{
-	  rvals[0].data.d_status = STATUS_EXECUTION_ERROR;
+	  rvals[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;
 	  break;
 	}
     break;
 
-    case RUN_WITH_LAST_VALS:
+    case GIMP_RUN_WITH_LAST_VALS:
       INIT_I18N();
       /* XXX: add code here for last-values running */
       if (pluginCore (&args, param[2].data.d_drawable) == -1)
 	{
-	  rvals[0].data.d_status = STATUS_EXECUTION_ERROR;
+	  rvals[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;
 	}
       break;
     }
@@ -249,8 +249,8 @@ pluginCore (struct piArgs *argp,
 	    gint32         drawable)
 {
   gint retval=0;
-  GDrawable *drw;
-  GPixelRgn srcPr, dstPr;
+  GimpDrawable *drw;
+  GimpPixelRgn srcPr, dstPr;
   guchar *src, *dst;
   guint width, height, Bpp;
 
@@ -373,10 +373,10 @@ pluginCoreIA (struct piArgs *argp,
 
   frame = gimp_radio_group_new2 (TRUE, _("Mode"),
 				 waves_radio_button_update,
-				 &argp->type, (gpointer) argp->type,
+				 &argp->type, GINT_TO_POINTER (argp->type),
 
-				 _("Smear"),   (gpointer) MODE_SMEAR, NULL,
-				 _("Blacken"), (gpointer) MODE_BLACKEN, NULL,
+				 _("Smear"),   GINT_TO_POINTER (MODE_SMEAR), NULL,
+				 _("Blacken"), GINT_TO_POINTER (MODE_BLACKEN), NULL,
 
 				 NULL);
   gtk_box_pack_start (GTK_BOX (hbox), frame, TRUE, TRUE, 0);
@@ -404,7 +404,7 @@ pluginCoreIA (struct piArgs *argp,
   table = gtk_table_new (3, 3, FALSE);
   gtk_table_set_col_spacings (GTK_TABLE (table), 4);
   gtk_table_set_row_spacings (GTK_TABLE (table), 2);
-  gtk_container_border_width (GTK_CONTAINER (table), 4);
+  gtk_container_set_border_width (GTK_CONTAINER (table), 4);
   gtk_container_add (GTK_CONTAINER (frame), table);
 
   adj = gimp_scale_entry_new (GTK_TABLE (table), 0, 0,
@@ -504,7 +504,7 @@ mw_preview_toggle_callback (GtkWidget *widget,
 }
 
 static struct mwPreview *
-mw_preview_build_virgin (GDrawable *drawable)
+mw_preview_build_virgin (GimpDrawable *drawable)
 {
   struct mwPreview *mwp;
 
@@ -530,13 +530,13 @@ mw_preview_build_virgin (GDrawable *drawable)
 }
 
 static struct mwPreview *
-mw_preview_build (GDrawable *drawable)
+mw_preview_build (GimpDrawable *drawable)
 {
   struct mwPreview *mwp;
   gint x, y, b;
   guchar *bc;
   guchar *drawableBits;
-  GPixelRgn pr;
+  GimpPixelRgn pr;
 
   mwp = mw_preview_build_virgin (drawable);
 
